import click


class Command:
    """Command wrapper wraps a CLI command definition the same way you would if you
    were to directly link it to the shell. Once you have accumulated all your
    commands, run `add_to_cli` to link them together.

    Example:
    ```
    cmd = Command()

    @cmd.command(name="doit)
    @click.pass_context
    def doit(ctx):
        print("hello world")

    @cmd.command(name="other)
    @click.pass_context
    def other(ctx):
        print("something els")

    def add_to_cli(cli):
        cmd.add_to_cli(cli)

    # Now in another module, where the shell is available, invoke add_to_cli against
    # it.
    ```
    """

    def __init__(self):
        self.commands = []

    def command(self, name: str, **kwargs):
        def decorator(function):
            command_factory = click.command(name=name, **kwargs)
            command = command_factory(function)
            self.commands.append(command)
            return command

        return decorator

    def add_to_cli(self, cli):
        for cmd in self.commands:
            cli.add_command(cmd)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.04.01
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.api_client import ApiClient, Endpoint as _Endpoint
from agilicus_api.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from agilicus_api.model.audit_destination import AuditDestination
from agilicus_api.model.audit_webhook_bulk_event import AuditWebhookBulkEvent
from agilicus_api.model.audit_webhook_events_processed import AuditWebhookEventsProcessed
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.list_audit_destinations_response import ListAuditDestinationsResponse
from agilicus_api.model.list_audits_response import ListAuditsResponse
from agilicus_api.model.list_auth_audits_response import ListAuthAuditsResponse


class AuditsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __bulk_create_events(
            self,
            audit_webhook_bulk_event,
            **kwargs
        ):
            """Process an AuditWebhookBulkEvent  # noqa: E501

            Processes an AuditWebhookBulkEvent by putting it into the audit event pipeline. Agilicus audit agents expect webhook destinations to obey the semantics of this API, though the URL at which to post AuditWebhookEvents may differ from this. That is, this API specification may be used to develop your own webhook for processing events. If the webhook wants to implement fine-grained processing of events, it should respond with the AuditWebhookEventProcessed response. Otherwise, the audit agent should assume that the response applies to all entries.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.bulk_create_events(audit_webhook_bulk_event, async_req=True)
            >>> result = thread.get()

            Args:
                audit_webhook_bulk_event (AuditWebhookBulkEvent):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                AuditWebhookEventsProcessed
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['audit_webhook_bulk_event'] = \
                audit_webhook_bulk_event
            return self.call_with_http_info(**kwargs)

        if self.bulk_create_events is None:
            self.bulk_create_events = _Endpoint(
                settings={
                    'response_type': (AuditWebhookEventsProcessed,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/bulk_audit_events',
                    'operation_id': 'bulk_create_events',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'audit_webhook_bulk_event',
                    ],
                    'required': [
                        'audit_webhook_bulk_event',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'audit_webhook_bulk_event':
                            (AuditWebhookBulkEvent,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'audit_webhook_bulk_event': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__bulk_create_events
            )

        def __create_audit_destination(
            self,
            audit_destination,
            **kwargs
        ):
            """Create an AuditDestination  # noqa: E501

            Creating a new AuditDestination allows administrators to connect their sources of audit events to somewhere they can be stored.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_audit_destination(audit_destination, async_req=True)
            >>> result = thread.get()

            Args:
                audit_destination (AuditDestination):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                AuditDestination
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['audit_destination'] = \
                audit_destination
            return self.call_with_http_info(**kwargs)

        if self.create_audit_destination is None:
            self.create_audit_destination = _Endpoint(
                settings={
                    'response_type': (AuditDestination,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audit_destinations',
                    'operation_id': 'create_audit_destination',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'audit_destination',
                    ],
                    'required': [
                        'audit_destination',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'audit_destination':
                            (AuditDestination,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'audit_destination': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_audit_destination
            )

        def __delete_audit_destination(
            self,
            destination_id,
            **kwargs
        ):
            """Remove an AuditDestination  # noqa: E501

            Remove an AuditDestination. After removal, the system will stop sending events to the destination.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_audit_destination(destination_id, async_req=True)
            >>> result = thread.get()

            Args:
                destination_id (str): destinaton id in path

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['destination_id'] = \
                destination_id
            return self.call_with_http_info(**kwargs)

        if self.delete_audit_destination is None:
            self.delete_audit_destination = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audit_destinations/{destination_id}',
                    'operation_id': 'delete_audit_destination',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'destination_id',
                        'org_id',
                    ],
                    'required': [
                        'destination_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'destination_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'destination_id': 'destination_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'destination_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_audit_destination
            )

        def __get_audit_destination(
            self,
            destination_id,
            **kwargs
        ):
            """Get a single AuditDestination  # noqa: E501

            Get the details of a single AuditDestination. Specify the id of the organisation which owns this resource to ensure you have permission.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_audit_destination(destination_id, async_req=True)
            >>> result = thread.get()

            Args:
                destination_id (str): destinaton id in path

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                AuditDestination
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['destination_id'] = \
                destination_id
            return self.call_with_http_info(**kwargs)

        if self.get_audit_destination is None:
            self.get_audit_destination = _Endpoint(
                settings={
                    'response_type': (AuditDestination,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audit_destinations/{destination_id}',
                    'operation_id': 'get_audit_destination',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'destination_id',
                        'org_id',
                    ],
                    'required': [
                        'destination_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'destination_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'destination_id': 'destination_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'destination_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_audit_destination
            )

        def __list_audit_destinations(
            self,
            **kwargs
        ):
            """View audit destinations  # noqa: E501

            View configured audit destinations  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_audit_destinations(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                org_id (str): Organisation Unique identifier. [optional]
                destination_type (str): Destinaton_type in query. Filters AuditDestinations based on their type. . [optional]
                name (str): Filters based on whether or not the items in the collection have the given name. . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListAuditDestinationsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_audit_destinations is None:
            self.list_audit_destinations = _Endpoint(
                settings={
                    'response_type': (ListAuditDestinationsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audit_destinations',
                    'operation_id': 'list_audit_destinations',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'org_id',
                        'destination_type',
                        'name',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'org_id':
                            (str,),
                        'destination_type':
                            (str,),
                        'name':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'org_id': 'org_id',
                        'destination_type': 'destination_type',
                        'name': 'name',
                    },
                    'location_map': {
                        'limit': 'query',
                        'org_id': 'query',
                        'destination_type': 'query',
                        'name': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_audit_destinations
            )

        def __list_audits(
            self,
            **kwargs
        ):
            """View audit records  # noqa: E501

            View audit records for any API.  The attribute_type, attribute_id and attribute_org_id query provides the ability to query attributes for an audit record across all possible attributes thay maybe associated with a record, as audit records may have multiple multiple attributes (AuditAttribute) per record.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_audits(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                user_id (str): Query based on user id. [optional]
                dt_from (str): Search criteria from when the query happened. * Inclusive. * In UTC. * Supports human-friendly values such as \"now\", \"today\", \"now-1day\". . [optional] if omitted the server will use the default value of ""
                dt_to (str): Search criteria until when the query happened. * Exclusive. * In UTC. * Supports human-friendly values such as \"now\", \"today\", \"now-1day\". . [optional] if omitted the server will use the default value of ""
                action (str): the type of action which caused the log. [optional] if omitted the server will use the default value of ""
                target_id (str): The identifier for the target of the log (e.g. the jti of a created token). . [optional] if omitted the server will use the default value of ""
                token_id (str): The id of the bearer token for which to find records.. [optional]
                api_name (str): The name of the API which generated the audit logs. [optional] if omitted the server will use the default value of ""
                target_resource_type (str): Filters the type of resource associated with the audit records.. [optional] if omitted the server will use the default value of ""
                org_id (str): Organisation Unique identifier. [optional]
                attribute_type (str): Filters the attribute_type associated with an audit record.. [optional]
                attribute_id (str): Filters the attribute_id associated with an audit record.. [optional]
                attribute_org_id (str): Filters the attribute_org_id associated with an audit record.. [optional]
                resources_behind_connector_id (str): search audit records for all resources behind a connector. [optional]
                attribute_type_list ([str]): a list of attribute_types for searching audit records.. [optional]
                attribute_id_list ([str]): a list of attribute_ids for searching audit records.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListAuditsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_audits is None:
            self.list_audits = _Endpoint(
                settings={
                    'response_type': (ListAuditsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audits',
                    'operation_id': 'list_audits',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'user_id',
                        'dt_from',
                        'dt_to',
                        'action',
                        'target_id',
                        'token_id',
                        'api_name',
                        'target_resource_type',
                        'org_id',
                        'attribute_type',
                        'attribute_id',
                        'attribute_org_id',
                        'resources_behind_connector_id',
                        'attribute_type_list',
                        'attribute_id_list',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'user_id':
                            (str,),
                        'dt_from':
                            (str,),
                        'dt_to':
                            (str,),
                        'action':
                            (str,),
                        'target_id':
                            (str,),
                        'token_id':
                            (str,),
                        'api_name':
                            (str,),
                        'target_resource_type':
                            (str,),
                        'org_id':
                            (str,),
                        'attribute_type':
                            (str,),
                        'attribute_id':
                            (str,),
                        'attribute_org_id':
                            (str,),
                        'resources_behind_connector_id':
                            (str,),
                        'attribute_type_list':
                            ([str],),
                        'attribute_id_list':
                            ([str],),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'user_id': 'user_id',
                        'dt_from': 'dt_from',
                        'dt_to': 'dt_to',
                        'action': 'action',
                        'target_id': 'target_id',
                        'token_id': 'token_id',
                        'api_name': 'api_name',
                        'target_resource_type': 'target_resource_type',
                        'org_id': 'org_id',
                        'attribute_type': 'attribute_type',
                        'attribute_id': 'attribute_id',
                        'attribute_org_id': 'attribute_org_id',
                        'resources_behind_connector_id': 'resources_behind_connector_id',
                        'attribute_type_list': 'attribute_type_list',
                        'attribute_id_list': 'attribute_id_list',
                    },
                    'location_map': {
                        'limit': 'query',
                        'user_id': 'query',
                        'dt_from': 'query',
                        'dt_to': 'query',
                        'action': 'query',
                        'target_id': 'query',
                        'token_id': 'query',
                        'api_name': 'query',
                        'target_resource_type': 'query',
                        'org_id': 'query',
                        'attribute_type': 'query',
                        'attribute_id': 'query',
                        'attribute_org_id': 'query',
                        'resources_behind_connector_id': 'query',
                        'attribute_type_list': 'query',
                        'attribute_id_list': 'query',
                    },
                    'collection_format_map': {
                        'attribute_type_list': 'multi',
                        'attribute_id_list': 'multi',
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_audits
            )

        def __list_auth_records(
            self,
            **kwargs
        ):
            """View authentication audit records  # noqa: E501

            View and search authentication audit records for different users and organisations in the system.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_auth_records(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                user_id (str): Query based on user id. [optional]
                dt_from (str): Search criteria from when the query happened. * Inclusive. * In UTC. * Supports human-friendly values such as \"now\", \"today\", \"now-1day\". . [optional] if omitted the server will use the default value of ""
                dt_to (str): Search criteria until when the query happened. * Exclusive. * In UTC. * Supports human-friendly values such as \"now\", \"today\", \"now-1day\". . [optional] if omitted the server will use the default value of ""
                org_id (str): Organisation Unique identifier. [optional]
                session_id (str): The session formed when the user started to log in.. [optional]
                trace_id (str): The id representing the request that triggered the event. [optional]
                upstream_user_id (str): The id of the user from upstream. [optional]
                upstream_idp (str): The name of the upstream idp. [optional]
                login_org_id (str): The org id the user tried to log in to. [optional]
                source_ip (str): The source IP address of the client device logging in.. [optional]
                client_id (str): The oidc client id used to log in. [optional]
                event (str): The event which triggered the audit record. [optional]
                stage (str): The stage of a pipeline to query for. [optional]
                request_id (str): The request id associated with an audit record . [optional]
                result (str): The result of an authentication audit to query.. [optional]
                event_name (str): query the event_name in authentication audit. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListAuthAuditsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_auth_records is None:
            self.list_auth_records = _Endpoint(
                settings={
                    'response_type': (ListAuthAuditsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/auth_audits',
                    'operation_id': 'list_auth_records',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'user_id',
                        'dt_from',
                        'dt_to',
                        'org_id',
                        'session_id',
                        'trace_id',
                        'upstream_user_id',
                        'upstream_idp',
                        'login_org_id',
                        'source_ip',
                        'client_id',
                        'event',
                        'stage',
                        'request_id',
                        'result',
                        'event_name',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'user_id':
                            (str,),
                        'dt_from':
                            (str,),
                        'dt_to':
                            (str,),
                        'org_id':
                            (str,),
                        'session_id':
                            (str,),
                        'trace_id':
                            (str,),
                        'upstream_user_id':
                            (str,),
                        'upstream_idp':
                            (str,),
                        'login_org_id':
                            (str,),
                        'source_ip':
                            (str,),
                        'client_id':
                            (str,),
                        'event':
                            (str,),
                        'stage':
                            (str,),
                        'request_id':
                            (str,),
                        'result':
                            (str,),
                        'event_name':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'user_id': 'user_id',
                        'dt_from': 'dt_from',
                        'dt_to': 'dt_to',
                        'org_id': 'org_id',
                        'session_id': 'session_id',
                        'trace_id': 'trace_id',
                        'upstream_user_id': 'upstream_user_id',
                        'upstream_idp': 'upstream_idp',
                        'login_org_id': 'login_org_id',
                        'source_ip': 'source_ip',
                        'client_id': 'client_id',
                        'event': 'event',
                        'stage': 'stage',
                        'request_id': 'request_id',
                        'result': 'result',
                        'event_name': 'event_name',
                    },
                    'location_map': {
                        'limit': 'query',
                        'user_id': 'query',
                        'dt_from': 'query',
                        'dt_to': 'query',
                        'org_id': 'query',
                        'session_id': 'query',
                        'trace_id': 'query',
                        'upstream_user_id': 'query',
                        'upstream_idp': 'query',
                        'login_org_id': 'query',
                        'source_ip': 'query',
                        'client_id': 'query',
                        'event': 'query',
                        'stage': 'query',
                        'request_id': 'query',
                        'result': 'query',
                        'event_name': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_auth_records
            )

        def __replace_audit_destination(
            self,
            destination_id,
            **kwargs
        ):
            """Create or update a AuditDestination.  # noqa: E501

            Create or update a AuditDestination.  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_audit_destination(destination_id, async_req=True)
            >>> result = thread.get()

            Args:
                destination_id (str): destinaton id in path

            Keyword Args:
                audit_destination (AuditDestination): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                AuditDestination
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['destination_id'] = \
                destination_id
            return self.call_with_http_info(**kwargs)

        if self.replace_audit_destination is None:
            self.replace_audit_destination = _Endpoint(
                settings={
                    'response_type': (AuditDestination,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/audit_destinations/{destination_id}',
                    'operation_id': 'replace_audit_destination',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'destination_id',
                        'audit_destination',
                    ],
                    'required': [
                        'destination_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'destination_id':
                            (str,),
                        'audit_destination':
                            (AuditDestination,),
                    },
                    'attribute_map': {
                        'destination_id': 'destination_id',
                    },
                    'location_map': {
                        'destination_id': 'path',
                        'audit_destination': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_audit_destination
            )

    bulk_create_events = None 
    create_audit_destination = None 
    delete_audit_destination = None 
    get_audit_destination = None 
    list_audit_destinations = None 
    list_audits = None 
    list_auth_records = None 
    replace_audit_destination = None 



from unittest.mock import MagicMock

class PolicyConfigApiMock:

    def __init__(self):
        self.mock_delete_authz_bundle = MagicMock()
        self.mock_delete_resource_url_bundle = MagicMock()
        self.mock_get_authz_bundle = MagicMock()
        self.mock_get_resource_url_bundle = MagicMock()

    def delete_authz_bundle(self, *args, **kwargs):
        """
        This method mocks the original api PolicyConfigApi.delete_authz_bundle with MagicMock.
        """
        return self.mock_delete_authz_bundle(self, *args, **kwargs)

    def delete_resource_url_bundle(self, *args, **kwargs):
        """
        This method mocks the original api PolicyConfigApi.delete_resource_url_bundle with MagicMock.
        """
        return self.mock_delete_resource_url_bundle(self, *args, **kwargs)

    def get_authz_bundle(self, *args, **kwargs):
        """
        This method mocks the original api PolicyConfigApi.get_authz_bundle with MagicMock.
        """
        return self.mock_get_authz_bundle(self, *args, **kwargs)

    def get_resource_url_bundle(self, *args, **kwargs):
        """
        This method mocks the original api PolicyConfigApi.get_resource_url_bundle with MagicMock.
        """
        return self.mock_get_resource_url_bundle(self, *args, **kwargs)




"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.04.01
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.regions_api import RegionsApi  # noqa: E501


class TestRegionsApi(unittest.TestCase):
    """RegionsApi unit test stubs"""

    def setUp(self):
        self.api = RegionsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_cluster(self):
        """Test case for add_cluster

        Add a Cluster  # noqa: E501
        """
        pass

    def test_add_point_of_presence(self):
        """Test case for add_point_of_presence

        Add a point of presence.  # noqa: E501
        """
        pass

    def test_add_region(self):
        """Test case for add_region

        Add a Region  # noqa: E501
        """
        pass

    def test_delete_cluster(self):
        """Test case for delete_cluster

        Delete a Cluster  # noqa: E501
        """
        pass

    def test_delete_point_of_presence(self):
        """Test case for delete_point_of_presence

        Delete a point of presence  # noqa: E501
        """
        pass

    def test_delete_region(self):
        """Test case for delete_region

        Delete a Region  # noqa: E501
        """
        pass

    def test_get_cluster(self):
        """Test case for get_cluster

        Get a Cluster  # noqa: E501
        """
        pass

    def test_get_point_of_presence(self):
        """Test case for get_point_of_presence

        Get a point of presence  # noqa: E501
        """
        pass

    def test_get_region(self):
        """Test case for get_region

        Get a Region  # noqa: E501
        """
        pass

    def test_list_clusters(self):
        """Test case for list_clusters

        List all Clusters  # noqa: E501
        """
        pass

    def test_list_point_of_presences(self):
        """Test case for list_point_of_presences

        List all regions  # noqa: E501
        """
        pass

    def test_list_regions(self):
        """Test case for list_regions

        List all regions  # noqa: E501
        """
        pass

    def test_replace_cluster(self):
        """Test case for replace_cluster

        update a Cluster  # noqa: E501
        """
        pass

    def test_replace_point_of_presence(self):
        """Test case for replace_point_of_presence

        update a point of presence  # noqa: E501
        """
        pass

    def test_replace_region(self):
        """Test case for replace_region

        update a Region  # noqa: E501
        """
        pass

    def test_routing_request(self):
        """Test case for routing_request

        Request routing  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from unittest.mock import MagicMock

class RegionsApiMock:

    def __init__(self):
        self.mock_add_cluster = MagicMock()
        self.mock_add_point_of_presence = MagicMock()
        self.mock_add_region = MagicMock()
        self.mock_delete_cluster = MagicMock()
        self.mock_delete_point_of_presence = MagicMock()
        self.mock_delete_region = MagicMock()
        self.mock_get_cluster = MagicMock()
        self.mock_get_point_of_presence = MagicMock()
        self.mock_get_region = MagicMock()
        self.mock_list_clusters = MagicMock()
        self.mock_list_point_of_presences = MagicMock()
        self.mock_list_regions = MagicMock()
        self.mock_replace_cluster = MagicMock()
        self.mock_replace_point_of_presence = MagicMock()
        self.mock_replace_region = MagicMock()
        self.mock_routing_request = MagicMock()

    def add_cluster(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.add_cluster with MagicMock.
        """
        return self.mock_add_cluster(self, *args, **kwargs)

    def add_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.add_point_of_presence with MagicMock.
        """
        return self.mock_add_point_of_presence(self, *args, **kwargs)

    def add_region(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.add_region with MagicMock.
        """
        return self.mock_add_region(self, *args, **kwargs)

    def delete_cluster(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.delete_cluster with MagicMock.
        """
        return self.mock_delete_cluster(self, *args, **kwargs)

    def delete_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.delete_point_of_presence with MagicMock.
        """
        return self.mock_delete_point_of_presence(self, *args, **kwargs)

    def delete_region(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.delete_region with MagicMock.
        """
        return self.mock_delete_region(self, *args, **kwargs)

    def get_cluster(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.get_cluster with MagicMock.
        """
        return self.mock_get_cluster(self, *args, **kwargs)

    def get_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.get_point_of_presence with MagicMock.
        """
        return self.mock_get_point_of_presence(self, *args, **kwargs)

    def get_region(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.get_region with MagicMock.
        """
        return self.mock_get_region(self, *args, **kwargs)

    def list_clusters(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.list_clusters with MagicMock.
        """
        return self.mock_list_clusters(self, *args, **kwargs)

    def list_point_of_presences(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.list_point_of_presences with MagicMock.
        """
        return self.mock_list_point_of_presences(self, *args, **kwargs)

    def list_regions(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.list_regions with MagicMock.
        """
        return self.mock_list_regions(self, *args, **kwargs)

    def replace_cluster(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.replace_cluster with MagicMock.
        """
        return self.mock_replace_cluster(self, *args, **kwargs)

    def replace_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.replace_point_of_presence with MagicMock.
        """
        return self.mock_replace_point_of_presence(self, *args, **kwargs)

    def replace_region(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.replace_region with MagicMock.
        """
        return self.mock_replace_region(self, *args, **kwargs)

    def routing_request(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.routing_request with MagicMock.
        """
        return self.mock_routing_request(self, *args, **kwargs)




from unittest.mock import MagicMock

class CataloguesApiMock:

    def __init__(self):
        self.mock_create_catalogue = MagicMock()
        self.mock_create_catalogue_entry = MagicMock()
        self.mock_delete_catalogue = MagicMock()
        self.mock_delete_catalogue_entry = MagicMock()
        self.mock_get_catalogue = MagicMock()
        self.mock_get_catalogue_entry = MagicMock()
        self.mock_list_all_catalogue_entries = MagicMock()
        self.mock_list_catalogue_entries = MagicMock()
        self.mock_list_catalogues = MagicMock()
        self.mock_replace_catalogue = MagicMock()
        self.mock_replace_catalogue_entry = MagicMock()

    def create_catalogue(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.create_catalogue with MagicMock.
        """
        return self.mock_create_catalogue(self, *args, **kwargs)

    def create_catalogue_entry(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.create_catalogue_entry with MagicMock.
        """
        return self.mock_create_catalogue_entry(self, *args, **kwargs)

    def delete_catalogue(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.delete_catalogue with MagicMock.
        """
        return self.mock_delete_catalogue(self, *args, **kwargs)

    def delete_catalogue_entry(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.delete_catalogue_entry with MagicMock.
        """
        return self.mock_delete_catalogue_entry(self, *args, **kwargs)

    def get_catalogue(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.get_catalogue with MagicMock.
        """
        return self.mock_get_catalogue(self, *args, **kwargs)

    def get_catalogue_entry(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.get_catalogue_entry with MagicMock.
        """
        return self.mock_get_catalogue_entry(self, *args, **kwargs)

    def list_all_catalogue_entries(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.list_all_catalogue_entries with MagicMock.
        """
        return self.mock_list_all_catalogue_entries(self, *args, **kwargs)

    def list_catalogue_entries(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.list_catalogue_entries with MagicMock.
        """
        return self.mock_list_catalogue_entries(self, *args, **kwargs)

    def list_catalogues(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.list_catalogues with MagicMock.
        """
        return self.mock_list_catalogues(self, *args, **kwargs)

    def replace_catalogue(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.replace_catalogue with MagicMock.
        """
        return self.mock_replace_catalogue(self, *args, **kwargs)

    def replace_catalogue_entry(self, *args, **kwargs):
        """
        This method mocks the original api CataloguesApi.replace_catalogue_entry with MagicMock.
        """
        return self.mock_replace_catalogue_entry(self, *args, **kwargs)




from . import context


def delete(ctx, **kwargs):
    """Delete each of the components of a demo setup."""
    """
     - application demo-container
     - desktop demo-desktop
     - share demo-share
     - application_service demo-container-local-service-0
     - service-account demo-container-service-account
     - connector demo-connector
    """
    apiclient = context.get_apiclient_from_ctx(ctx)

    query_results = apiclient.connectors_api.list_agent_connector(**kwargs)
    for app in query_results.agent_connectors:
        if app.spec.name == "demo-connector":
            print(f"Will delete connector {app.metadata.id}")
            apiclient.connectors_api.delete_connector(
                **kwargs, connector_id=app.metadata.id
            )

    query_results = apiclient.application_api.list_applications(**kwargs)
    for app in query_results.applications:
        if app.name == "demo-container":
            print(f"Will delete application {app.id}")
            apiclient.application_api.delete_application(**kwargs, app_id=app.id)

    query_results = apiclient.app_services_api.list_desktop_resources(**kwargs)
    for app in query_results.desktop_resources:
        if app.spec.name == "demo-desktop":
            print(f"Will delete desktop {app.metadata.id}")
            apiclient.app_services_api.delete_desktop_resource(
                **kwargs, resource_id=app.metadata.id
            )

    query_results = apiclient.app_services_api.list_file_share_services(**kwargs)
    for app in query_results.file_share_services:
        if app.spec.name == "demo-share":
            print(f"Will delete share {app.metadata.id}")
            apiclient.app_services_api.delete_file_share_service(
                **kwargs, file_share_service_id=app.metadata.id
            )

    query_results = apiclient.app_services_api.list_application_services(**kwargs)
    for app in query_results.application_services:
        if app.name == "demo-container-local-service-0":
            print(f"Will delete app-serv {app.id}")
            apiclient.app_services_api.delete_application_service(
                **kwargs, app_service_id=app.id
            )

    query_results = apiclient.user_api.list_service_accounts(**kwargs)
    for sa in query_results.service_accounts:
        if sa.spec.name == "agent-connector-demo-connector-service-account":
            print(f"Will delete service-account {sa.metadata.id}")
            apiclient.user_api.delete_service_account(
                **kwargs, service_account_id=sa.metadata.id
            )



import agilicus
from ..context import get_apiclient_from_ctx
import operator
from agilicus.input_helpers import pop_item_if_none
from agilicus.input_helpers import update_attrs_if_not_none
from ..output.table import (
    spec_column,
    metadata_column,
    format_table,
    subtable,
)


def get_api(ctx):
    return get_apiclient_from_ctx(ctx).billing_api


def list_features(ctx, **kwargs):
    api = get_api(ctx)
    return api.list_features(**kwargs).features


def format_features(ctx, features):
    product_columns = [
        spec_column("name"),
        spec_column("label", optional=True),
        spec_column("description", optional=True),
    ]
    columns = [
        metadata_column("id"),
        spec_column("name"),
        spec_column("key"),
        spec_column("value"),
        subtable(
            ctx,
            "products",
            columns=product_columns,
            subobject_name="status",
            table_getter=operator.itemgetter,
        ),
    ]
    return format_table(ctx, features, columns)


def add_feature(
    ctx,
    min=None,
    max=None,
    enabled=None,
    priority=None,
    **kwargs,
):
    api = get_api(ctx)
    pop_item_if_none(kwargs)
    spec = agilicus.FeatureSpec(**kwargs)
    val = agilicus.FeatureValue()
    spec.value = val
    if min:
        val.min = min

    if max:
        val.max = max

    if enabled is not None:
        val.enabled = enabled

    if priority is not None:
        spec.priority = priority
    return api.create_feature(agilicus.Feature(spec))


def delete_feature(ctx, **kwargs):
    api = get_api(ctx)
    return api.delete_feature(**kwargs)


def list_feature_subscriptions(ctx, **kwargs):
    api = get_api(ctx)
    return api.list_subscriptions_with_feature(**kwargs).billing_subscriptions


def update_feature(
    ctx,
    feature_id,
    **kwargs,
):
    api = get_api(ctx)

    feature = api.get_feature(feature_id)

    pop_item_if_none(kwargs)

    vals = {}
    for v in ["min", "max", "enabled"]:
        vals[v] = kwargs.pop(v, None)

    pop_item_if_none(vals)
    if not feature.spec.value:
        feature.spec.value = agilicus.FeatureValue()

    update_attrs_if_not_none(feature.spec, kwargs)
    update_attrs_if_not_none(feature.spec.value, vals)

    return api.replace_feature(feature_id, feature=feature)
